/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 6 (Figure 1.1)
**
*/

new;
library rpb,pgraph;
rpbSet;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

c = 5;
Bounds = (-c~c) .* ones(n,1);
nS = 1000;

call init_Sobol(n);
u = rndu_Sobol(nS,n);
u = 2*u - 1;
u = c * u;
u = u ./ sumr(u);

mu_s = zeros(nS,1);
sigma_s = zeros(nS,1);

i = 1;
do until i > nS;
  x = u[i,.]';
  mu_s[i] = mu'x;
  sigma_s[i] = sqrt(x'cov*x);
  i = i + 1;
endo;

{mu_s,sigma_s} = packr2(mu_s,sigma_s);
nS = rows(mu_S);

gamma_ = seqa(-2,1/100,600);
phi = 1/gamma_;
nPhi = rows(phi);
{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
mu0 = rndu(nPhi,1);
sigma0 = rndu(nPhi,1);

nP = _max_(nS,nPhi);
all_mu = miss(zeros(nP,2),0);
all_sigma = miss(zeros(nP,2),0);
all_mu[1:nPhi,1] = -mu0;
all_mu[1:nS,2] = mu_s;
all_sigma[1:nPhi,1] = sigma0;
all_sigma[1:nS,2] = sigma_s;

{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.30,2);  
{x2,mu2,sigma2,gamma2,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.07,1); 

{x3,mu3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.26,2); 
{x4,mu4,sigma4,gamma4,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.06,1); 

/*
indx_CH = ConvexHull(sigma_s~mu_s); 
save results\chap1-1-1-1 = indx_CH;
*/

loadm indx_CH = results\chap1-1-1-1; // loads the convex hull points
nCH = rows(indx_CH);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1|4;
  _plctrl = 0|-1; _pstype = 4;
  xtics(0,40,5,2);
  ytics(0,12,2,0);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  _psym = 100*sigma_s[indx_CH]~100*mu_s[indx_CH]~ones(nCH,1) .* (8~2.5~4~1~4) |
          100*sigma1~100*mu1~8~3.5~0~1~4 |
          100*sigma2~100*mu2~8~3.5~5~1~4 |
          100*sigma1~8~8~3.5~0~1~4 |
          100*sigma3~100*mu3~8~3.5~0~1~4 |
          20~100*mu2~8~3.5~5~1~4 |
          100*sigma4~100*mu4~8~3.5~5~1~4 ;
  _pline = 1~1~0~100*mu1~35~100*mu1~1~0~10 |
           1~1~100*sigma1~0~100*sigma1~12~1~0~10 |
           1~3~0~100*mu2~22~100*mu2~1~5~10 |
           1~3~100*sigma2~0~100*sigma2~9~1~5~10 ;
  _pmsgstr = "A\000B\000C\000D\000E\000F";
  _pmsgctl =   100*(sigma1+0.003)~100*(mu1+0.002)~0.17~0~1~0~8 |
               100*(sigma3+0.003)~100*(mu3+0.002)~0.17~0~1~0~8 |
               100*(sigma1+0.003)~100*(0.08+0.002)~0.17~0~1~0~8 |
               100*(sigma2-0.01)~100*(mu2+0.002)~0.17~0~1~5~8 |
               100*(sigma4-0.01)~100*(mu4+0.002)~0.17~0~1~5~8 |
               100*(0.20-0.01)~100*(mu2+0.002)~0.17~0~1~5~8 ;
  graphprt("-c=1 -cf=chap1-1-1-1.eps");
  xy(100*all_sigma,100*all_mu);