/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 46 (Resampling techniques)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.09 0.07 0.06;
let sigma = 0.04 0.15 0.05 0.10;
let rho =   1.00 
            0.10  1.00
            0.40  0.20  1.00
           -0.10 -0.10 -0.20 1.00;
rho = xpnd(rho);
Cov = rho .* sigma .* sigma';
n = rows(mu);

Bounds = 0~1;

vol_target = seqa(3.4,3/10,11)/100;
nP = rows(vol_target);

gamma_ = seqa(-1,0.005,300);
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,gamma_,0);
{x_star,er_star,risk_star,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,vol_target,2);

nS = 100;
all_er = zeros(nP,nS);
all_risk = zeros(nP,nS);
all_x = zeros(n,nP);

rndseed 123;

i = 1;
do until i > nS;
  r = rndmn(mu,Cov,260);
  c = vcx(r);
  m = meanc(r);
  {x,all_er[.,i],all_risk[.,i],gamma_x,retcode} = Compute_MVO_Portfolio(m,c,1,0,0,0,Bounds,vol_target,2);
  all_x = all_x + x;
  i = i + 1;
endo;

x_star_Resampling = all_x / nS;
er_star_Resampling = meanc(all_er');
risk_star_Resampling = meanc(all_risk');

Results = x_star|er_star'|risk_star';
print ftosa(100*Results,"%lf",3,2);

Results = x_star_Resampling|er_star_Resampling'|risk_star_Resampling';
print ftosa(100*Results,"%lf",3,2);

all_er = vecr(all_er);
all_risk = vecr(all_risk);
np = rows(all_er);

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected Return (in %)");
  xtics(3,6,0.5,2);
  ytics(5,7.5,0.5,2);

  setwind(1);
    _psym = 100*(all_risk~all_er)~ones(np,1).*(11~2~3~1~2);
    xy(100*Risk,100*ER);

  setwind(2);
    _psym = 0; _pcolor = 0|0; _plwidth = 12;
    xy(100*Risk,100*ER);

  endwind;

