/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 94 and 95
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

theta = 2;

m1 = 0.10; s1 = 0.20; n1 = 6; x1 = 0.60;
m2 = 0.10; s2 = 0.25; n2 = 4; x2 = 0.40;
x = x1|x2;

n = rows(x);
nS = 1000;

rho = 0.5;
{u1,u2} = rndCopulaClayton(theta,nS);
R1 = m1 + s1 * cdfti(u1,n1);
R2 = m2 + s2 * cdfti(u2,n2);
R = R1~R2;
L = -(x1 .* R1 + x2 .* R2);

moments = zeros(4,1);
moments[1] = meanc(L);
moments[2] = (nS-1)/nS*stdc(L)^2;
moments[3] = Estimate_Skewness(L);
moments[4] = Estimate_Kurtosis(L);

Sigma = (nS - 1) * vcx(R1~R2) / nS;

Rc = R - meanc(R)';

M1 = meanc(R);
M2 = 0;
M3 = 0;
M4 = 0;

i = 1;
do until i > nS;
  y = Rc[i,.]';
  M2 = M2 + (y * y')/nS;
  M3 = M3 + ((y * y') .*. y')/nS;
  M4 = M4 + ((y * y') .*. y' .*. y')/nS;
  i = i + 1;
endo;

mu = zeros(4,1);
mu[1] = x'M1;
mu[2] = x'M2*x;
mu[3] = x'M3*(x .*. x);
mu[4] = x'M4*(x .*. x .*. x);

portfolio_moments = zeros(4,1);
portfolio_moments[1] = -mu[1];
portfolio_moments[2] = mu[2];
portfolio_moments[3] = -mu[3] / (mu[2]^1.5);
portfolio_moments[4] = mu[4] / (mu[2]^2) - 3;

Results = moments~portfolio_moments;

print ftosa(Results,"%lf",3,7);
print "------------------------------";

alpha = 0.99;
gVaR = moments[1] + cdfni(alpha) * sqrt(moments[2]);
gRC = x .* (-M1 + cdfni(alpha) * (M2*x)  / sqrt(moments[2]));

{mVaR,mRC,databuf} = Compute_Cornish_Fisher_VaR(x,R,alpha);

m_moments = vread(databuf,"moments");
m_mu = vread(databuf,"mu");
m_gamma = vread(databuf,"gamma");
d_gamma = vread(databuf,"d_gamma");
z = vread(databuf,"z");
d_z = vread(databuf,"d_z");

Results = m_moments ~ (m_mu[1:2]|m_gamma);

print ftosa(Results,"%lf",3,7);
print "------------------------------";

d_x = 0.000001;
new_mRC = zeros(n,1);
new_d_gamma = zeros(n,2);
new_d_z = zeros(n,1);

i = 1;
do until i > n;
  new_x = x;
  new_x[i] = new_x[i] + d_x;
  {new_mVaR,new_mRC,new_databuf} = Compute_Cornish_Fisher_VaR(new_x,R,alpha);
  new_m_gamma = vread(new_databuf,"gamma");
  new_z = vread(new_databuf,"z");

  new_mRC[i] = x[i] * (new_mVaR - mVaR)/d_x;
  new_d_gamma[i,.] = (new_m_gamma' - m_gamma')/d_x;
  new_d_z[i] = (new_z - z)/d_x;

  i = i + 1;
endo;

Results = (gVaR|gRC|sumc(gRC))~(mVaR|mRC|sumc(mRC))~(mVaR|new_mRC|sumc(new_mRC));
print ftosa(Results,"%lf",3,7);
print "------------------------------";

Results = (m_gamma'|d_gamma)~(new_m_gamma'|new_d_gamma);
print ftosa(Results,"%lf",3,7);
print "------------------------------";

Results = d_z~new_d_z;
print ftosa(Results,"%lf",3,7);
print "------------------------------";

