/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 96 & 97 (Table 2.7 & Figure 2.3)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

nS = 1000000;

x = seqa(-0.40,0.70/250,251);
nx = rows(x);

let xi  = 0.01 -0.05   0.15;
let omega = 0.05  0.10   0.20;
let beta  = 0.00  2.00  -5.00;
nParams = rows(beta);

pdf = zeros(nx,nParams);
moments = zeros(4,nParams);

let alpha = 0.80 0.85 0.90 0.95 0.99;
q = zeros(rows(alpha),nParams);

i = 1;
do until i > nParams;
  pdf[.,i] = pdfSN(x,xi[i],omega[i],beta[i]);
  PnL = rndSN(nS,1,xi[i],omega[i],beta[i]);
  q[.,i] = -cdfSNi(1-alpha,xi[i],omega[i],beta[i]);
  L = -PnL;
  moments[1,i] = meanc(L);
  moments[2,i] = (nS-1)/nS*stdc(L)^2;
  moments[3,i] = Estimate_Skewness(L);
  moments[4,i] = Estimate_Kurtosis(L) - 3;
  i = i + 1;
endo;

Loss_moments = moments;

delta = beta ./ sqrt(1 + beta^2);
mu = delta * sqrt(2/pi);
mean = xi + omega .* mu;
variance = (omega^2) .* (1 - mu^2);
gamma1 = (4 - pi) / 2 * (mu^3) ./ ((1 - mu^2)^(3/2));
gamma2 = 2 * (pi - 3) * (mu^4) ./ ((1 - mu^2)^2);
moments_Pi = mean'|variance'|gamma1'|gamma2';
moments_L = -mean'|variance'|-gamma1'|gamma2';

print ftosa(moments_Pi',"%lf",3,5);
print "------------------------------";
print ftosa(moments_L,"%lf",3,5);
print "------------------------------";
print ftosa(loss_moments,"%lf",3,5);
print "------------------------------";

z = zeros(rows(alpha),nParams);
VaR = zeros(rows(alpha),nParams);
gVaR = zeros(rows(alpha),nParams);

i = 1;
do until i > nParams;
  mu_L = -mean[i];
  sigma_L = sqrt(variance[i]);
  gamma1_L = -gamma1[i];
  gamma2_L = gamma2[i];
  {z[.,i],gVaR[.,i]} = Compute_Cornish_Fisher_z(mu_L,sigma_L,0,0,alpha);
  {z[.,i],VaR[.,i]} = Compute_Cornish_Fisher_z(mu_L,sigma_L,gamma1_L,gamma2_L,alpha);
  i = i + 1;
endo;

print ftosa(100*gVaR',"%lf",3,2);
print "------------------------------";
print ftosa(100*q',"%lf",3,2);
print "------------------------------";
print ftosa(100*VaR',"%lf",3,2);
print "------------------------------";

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|1|3; _pcolor = 1|2|4; _plwidth = 10;
  xtics(-40,30,10,2);
  xlabel("\216\202P\201 (in %)");
  _plegstr = "#1\000#2\000#3";
  _plegctl = {2 5 1.5 5};
  graphprt("-c=1 -cf=chap2-1-3-9.eps");
  xy(100*x,pdf);



