/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 121 (Table 2.20)
**
*/

new;
library rpb,pgraph;

cls;

let sigma = 0.20 0.30;
rho = 0.50;
cov = diagrv(rho*ones(2,2),1) .* sigma .* sigma';
x0 =ones(2,1)/2;

k = sigma[2]/sigma[1];

Bounds = (-3~3) .* ones(2,1);
{x_min,vol_min,retcode} = Compute_MV_Portfolio(cov,ones(1,2),1,0,0,Bounds);
{vol_min,mr_min,rc_min,prc_min} = Compute_Risk_Contribution(x_min,cov);
result1 = 100*(x_min~mr_min~rc_min~prc_min) |
          100*(error(0)~error(0)~vol_min~error(0));

vol_min = k * sigma[1] * sqrt( (1 - rho^2) / (1 + k^2 - 2*rho*k));
x_min = (sigma[2]^2 - rho*sigma[2]*sigma[1]) | (sigma[1]^2 - rho*sigma[2]*sigma[1]);
x_min = x_min / (sigma[1]^2 + sigma[2]^2 - 2*rho*sigma[2]*sigma[1]);

{x_erc,vol_erc,mr_erc,rc_erc,prc_erc} = Compute_ERC_Portfolio(x0,cov);
result2 = 100*(x_erc~mr_erc~rc_erc~prc_erc) |
          100*(error(0)~error(0)~vol_erc~error(0));
vol_erc = sqrt(2*(1+rho))*k/(1+k)*sigma[1];

x_ew = ones(2,1)/2;
{vol_ew,mr_ew,rc_ew,prc_ew} = Compute_Risk_Contribution(x_ew,cov);
result3 = 100*(x_ew~mr_ew~rc_ew~prc_ew) |
          100*(error(0)~error(0)~vol_ew~error(0));
vol_ew = 0.5 * sigma[1] * sqrt(1 + k^2 + 2*rho*k);

cls;

print ftosa(result3,"%lf",3,2);
print ftosa(result2,"%lf",3,2);
print ftosa(result1,"%lf",3,2);
print;
print ftosa(100*vol_ew,"vol(ew) =  %lf",3,2);
print ftosa(100*vol_erc,"vol(erc) = %lf",3,2);
print ftosa(100*vol_min,"vol(min) = %lf",3,2);
