/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 143 (Tables 2.29 and 2.30)
**
*/

new;

// #define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

Let A[4,3] = 0.9    0.0     0.5
             1.1    0.5     0.0
             1.2    0.3     0.2
             0.8    0.1     0.7;

let vol_F = 0.20 0.10 0.10;
Omega = diagrv(eye(3),vol_F^2);
let sigma_A = 0.10 0.15 0.10 0.15;
D = diagrv(eye(4),sigma_A^2);

Sigma = A*Omega*A' + D;
vol_A = sqrt(diag(Sigma));
rho_A = Sigma ./ vol_A ./ vol_A';

n = rows(A);
m = cols(A);

print ftosa(100*vol_A,"%lf",3,2);
print ftosa(100*rho_A,"%lf",3,2);

print "----------------------------------------------";

A_plus = pinv(A);
A_plus_check = invpd(A'A)*A';
B = A';
B_plus = pinv(B);
B_plus_check = B'*invpd(B*B');
B_tilde =  pinv(null(B_plus')) * (eye(n) - B_plus*A');

print "A+ = ";
print ftosa(A_plus,"%lf",3,3);
print ftosa(A_plus_check,"%lf",3,3);
print "B+ = ";
print ftosa(B_plus,"%lf",3,3);
print ftosa(B_plus_check,"%lf",3,3);
print "B_tilde = ";
print ftosa(B_tilde,"%lf",3,3);

let x = 0.25 0.25 0.25 0.25;

{vol,mr,rc,prc} = Compute_Risk_Contribution(x,Sigma);

print;
print "------------ RC (Assets) --------------------";
print (ftosa(100*vol,"vol = %lf",5,2) $+ ftosa(100*sumc(rc)," / tr = %lf",5,2));
print (ftosa(100*(x~mr~rc),"%lf",3,2) $~ ftosa(100*(rc/vol),"%lf",3,2));

{vol,y,mr,rc,prc} = Compute_RC_Factor(x,Sigma,A,Omega,D);

print "------------ RC (Factors) --------------------";
print (ftosa(100*vol,"vol = %lf",5,2) $+ ftosa(100*sumc(rc)," / tr = %lf",5,2));
print (ftosa(100*(y~mr~rc),"%lf",3,2) $~ ftosa(100*(rc/vol),"%lf",3,2));