/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 146-147 (Solving the polico invariance property)
**
*/

new;

#define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

let sigma[4,1] = 0.20 0.10 0.15 0.30;
n = rows(sigma);
let rho = 1.0
          0.5 1.0
          0.25 0.15 1.0
          0.10 0.30 0.40 1.0;
rho = xpnd(rho);
cov1 = rho .* sigma .* sigma';

let b1 = 0.25 0.25 0.30 0.20;
{x1,vol1,mr1,rc1,prc1} = Compute_RB_Portfolio(0,cov1,b1);

let c = 0.2 0.30 0.40 0.10;
c = c / sumc(c);

A = eye(n)|c';
Omega = cov1;
D = zeros(n+1,n+1);
cov2 = A*omega*A' + D;

b2 = b1;
indx = seqa(1,1,n);

_rpb_factor_bounds = -1;

sv = x1 | 0;
{x2,y2,f_mr2,f_rc2,f_prc2} = Compute_RB_Factor(sv,cov2,A,Omega,D,b2,indx,1); 

x3 = trimr(x2,0,1) + c .* x2[n+1];
{vol3,mr3,rc3,prc3} = Compute_Risk_Contribution(x3,cov1);

cls;

print "------------------------------------------------------";

print ftosa(100*vol1,"vol = %lf",3,2);
print ftosa(100*(x1~mr1~rc1~prc1),"%lf",3,2);

print "------------------------------------------------------";

print ftos(100*vol3,"vol = %lf",3,2);
print ftosa(100*(x3~mr3~rc3~prc3),"%lf",3,2);