/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 238 & 240 (Table 4.25 & Figure 4.22)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "Results\\sovereign-bonds-egbi.fmt";
loadm egbi = ^FileName;

FileName = "Results\\sovereign-bonds-rb-egbi.fmt";
loadm egbi_rb = ^FileName;

FileName = "Results\\sovereign-bonds-debt.fmt";
loadm debt = ^FileName;

FileName = "Results\\sovereign-bonds-rb-debt.fmt";
loadm debt_rb = ^FileName;

FileName = "Results\\sovereign-bonds-gdp.fmt";
loadm gdp = ^FileName;

FileName = "Results\\sovereign-bonds-rb-gdp.fmt";
loadm gdp_rb = ^FileName;

FileName = "Data//sovereign-bonds.fmt";
loadm databuf = ^FileName;
Dates = vread(databuf,"dates");
Spread = vread(databuf,"spread");
Duration = vread(databuf,"duration");
Weights_EGBI = vread(databuf,"w(egbi)");
Weights_GDP = vread(databuf,"w(gdp)");
Weights_DEBT = vread(databuf,"w(debt)");
EGBI_index = vread(databuf,"index");
EGBI_Subindex = vread(databuf,"subindex");
Libor = vread(databuf,"libor");

Weights_Dates = egbi[.,1];
nCountries = 11;

indx = indnv(Weights_Dates,Dates);
EGBI_Index = EGBI_Index[indx];
EGBI_SubIndex = EGBI_SubIndex[indx,.];
Libor = Libor[indx,.];

nCountries = 11;
indx_Weights = seqa(2,1,nCountries);

Weights_DEBT = DEBT[.,indx_Weights];
Weights_GDP = GDP[.,indx_Weights];
Weights_DEBT_RB = DEBT_RB[.,indx_Weights];
Weights_GDP_RB = GDP_RB[.,indx_Weights];
Weights_EGBI = EGBI[.,indx_Weights];

Dates = Weights_Dates;
nDates = rows(Dates);

nCols = 6;
Backtests = zeros(nDates,nCols);
Backtests[.,1] = EGBI_Index;
Backtests[.,2] = Simulate_Monthly_Backtest(Dates,Weights_DEBT,EGBI_SubIndex,0,0);
Backtests[.,3] = Simulate_Monthly_Backtest(Dates,Weights_GDP,EGBI_SubIndex,0,0);
Backtests[.,4] = Simulate_Monthly_Backtest(Dates,Weights_DEBT_RB,EGBI_SubIndex,0,0);
Backtests[.,5] = Simulate_Monthly_Backtest(Dates,Weights_GDP_RB,EGBI_SubIndex,0,0);
Backtests[.,6] = Simulate_Monthly_Backtest(Dates,Weights_EGBI,EGBI_SubIndex,0,0);
Libor_Index = Capitalized_Libor(Dates,Libor);

Backtests = 100 * Backtests ./ Backtests[1,.];

Rdt = packr(Backtests./lag1(Backtests) - 1.0);
stats = zeros(8,nCols);
indx_bench = 1;

i = 1;
do until i > nCols;
  Report = Reporting_Backtest(Dates,Backtests[.,i],Backtests[.,indx_bench],Libor_Index,0,0);
  mu = vread(Report,"mu");
  sigma = vread(Report,"sigma");
  sh = vread(Report,"SR");
  te = vread(Report,"sigma_te");
  ir = vread(Report,"ir");
  dd = vread(Report,"mdd");
  rho = submat(corrx(Rdt[.,indx_bench~i]),1,2);
  beta = rho * sigma / vread(Report,"sigma_B");
  stats[.,i] = 100*mu|100*sigma|sh|100*dd|100*te|ir|100*rho|beta;
  i = i + 1;
endo;

cls;

print ftosa(stats,"%lf",3,2);

Backtests[.,1] = missex(Backtests[.,1],Dates[.,1] .< 20100630);

data = Backtests[.,2 3 4 5 1];
t = seqa(1,1,nDates);
t = seqa(-60,1,60)|t;
data = miss(zeros(60,5),0) | data;

t = seqa(1,1,nDates);
t = seqa(-60,1,60)|t;
data = miss(zeros(60,5),0) | data;

step = 5;
if step == 1;
  indx = seqa(1,1,rows(t));
else;
  indx = seqa(1,step,rows(t)/step)|rows(t);
endif;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; _pgrid = {1,1};
  _pltype = 6|3|6|3|1; _plwidth = 10; _pcolor = 1|2|3|4|7; _plctrl = 0|0|131|131; _pstype = 8;  
  _plctrl = _plctrl/step;
  let xlab = "2008" "" "2009" "" "2010" "" "2011" "" "2012" "";
  asclabel(xlab,0);
  xtics(1,nDates,131,0);
  ytics(95,135,5,2);
  _plegstr = "DEBT-WB\000GDP-WB\000DEBT-RB\000GDP-RB\000EGBI";
  _plegctl = {2 5 1 5};
  graphprt("-c=1 -cf=chap4-3-2-19.eps");
  xy(t[indx],data[indx,.]);
