/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 228 & 229 (Tables 4.16, 4.17, 4.18 & 4.19)
**
*/

new;
library rpb,pgraph;

cls;

let S     =   76     166    356     356;
S = S * 1e-4;
let Sigma =   0.660  0.709  0.742   0.742;
let rho   =   1.00   
              0.86   1.00
              0.73   0.80   1.00
              0.73   0.80   1.00    1.00;
rho = xpnd(rho);

let x = 12  15     16      8;
let D = 8.2 7.1    6.5     5.9;

sigma_c = D .* Sigma .* S;
print ftosa(x~D~100*sigma_c,"%lf",3,2);

n1 = x[1:3];
x1 = n1 / sumc(n1);
n1 = sumc(n1);

n2 = x[1:4];
x2 = n2 / sumc(n2);
n2 = sumc(n2);

cov = rho .* sigma_c .* sigma_c';
cov1 = cov[1:3,1:3];
cov2 = cov;

R1 = sqrt(x1'cov1*x1);
R2 = sqrt(x2'cov2*x2);

print ftosa(n1*R1,"R1 = %lf",3,4);
print ftosa(n2*R2,"R2 = %lf",3,4);

{vol,mr,rc,prc} = Compute_Risk_Contribution(x1,cov1);
result1 = (x1~mr~rc~prc)|(error(0)~error(0)~vol~error(0));

{vol,mr,rc,prc} = Compute_Risk_Contribution(x2,cov2);
result2 = (x2~mr~rc~prc)|(error(0)~error(0)~vol~error(0));

print ftosa(100*result1,"%lf",3,2);
print ftosa(100*result2,"%lf",3,2);

n3 = x[1:2]|sumc(x[3:4]);
x3 = n3 / sumc(n3);
n3 = sumc(n3);
D[3] = (x[3] * D[3] + x[4] * D[4])/(x[3] + x[4]);
sigma_c = D .* Sigma .* S;
sigma_c = sigma_c[1:3];
cov3 = rho[1:3,1:3] .* sigma_c .* sigma_c';
{vol,mr,rc,prc} = Compute_Risk_Contribution(x3,cov3);
result3 = (x3~mr~rc~prc)|(error(0)~error(0)~vol~error(0));

print ftosa(100*result3,"%lf",3,2);


 


