/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 289 & 290 (Table 6.5 & Figure 6.12)
**
*/

new;
library rpb,pgraph;

cls;

//       US10Y  EUR10Y  IG    HY    US    EURO  JPY   EM    COMMO  
let indx =  1     2     3     4     5     6     7     8      9;  
let mu    = 4.2   3.8   5.3   10.4   9.2   8.6   5.3  11.0   8.8;
let sigma = 5.0   5.0   7.0   10.0  15.0  15.0  15.0  18.0  30.0;
let rho   = 1.00                                                  // US 10Y
            0.80  1.00                                            // EUR 10Y
            0.60  0.40  1.00                                      // IG
           -0.20 -0.20  0.50  1.00                                // HY

           -0.10 -0.20  0.30  0.60  1.00                          // US
           -0.20 -0.10  0.20  0.60  0.90  1.00                    // EURO
           -0.20 -0.20  0.20  0.50  0.70  0.60  1.00              // JPY
           -0.20 -0.20  0.30  0.60  0.70  0.70  0.70  1.00        // EM

            0.00  0.00  0.10  0.20  0.20  0.20  0.30  0.30  1.00;  // COMMO

mu = mu / 100;
sigma = sigma / 100;                                    
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
call invpd(cov);

let indx = 1    2    3     5     6    8      9;
let b    = 0.20 0.10 0.15  0.20  0.10 0.15 0.10;
cov = cov[indx,indx];
mu = mu[indx];
sigma = sigma[indx];
n = rows(indx);

{x0,vol0,mr0,rc0,prc0} = Compute_RB_Portfolio(0,cov,b);
er0 = x0'mu;

te = 0.01;
{x1,alpha1,te1,gamma1,retcode} = Compute_TE_Portfolio(x0,mu,cov,ones(1,n),1,0,0,(0~1).*ones(n,1),te,2);
{vol1,mr1,rc1,prc1} = Compute_Risk_Contribution(x1,cov);
er1 = x1'mu;

{x2,er2,vol2,gamma2,retcode} = Compute_MVO_Portfolio(mu,cov,ones(1,n),1,0,0,(0~1).*ones(n,1),vol1,2);
{vol2,mr2,rc2,prc2} = Compute_Risk_Contribution(x2,cov);

results = x0~prc0~x1~prc1~x2~prc2 |
          (vol0~error(0)~vol1~error(0)~vol2~error(0)) |
          (er0~error(0)~er1~error(0)~er2~error(0)) ;
print ftosa(100*Results,"%lf",3,1);

gamma_ = -3|seqa(-1.5,0.01,251);
{x,er,vol,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,ones(1,n),1,0,0,(0~1).*ones(n,1),gamma_,0);

vol_x = vol0 | vol1 | vol2;
er_x = er0 | er1 | er2;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xtics(3,9,1,2);
  ytics(3,8,1,2);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  e = ones(rows(vol_x),1);
  _psym = 100*(vol_x~er_x)~e .*(11~4)~(2|3|4)~e.*(1~10);
  _pmsgstr = "RB\000RB[*]\000MVO";
  let dx =  -0.20  0.05 -0.50;
  let dy =  -0.30 -0.25  0.10;
  _pmsgctl = (100*vol_x+dx)~(100*er_x+dy)~e .*(0.20~0~1)~(2|3|4)~e.*8;
  graphprt("-c=1 -cf=chap6-2-2-1.eps");
  xy(100*vol,100*er);



